%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% =======================================================
% Title:  PFT: a Protocol For evaluating video Trackers
% =======================================================
%
% Created by: Tahir Nawaz
% Supervisor: Andrea Cavallaro
% Organization: School of Electronic Engineering and Computer Science
%               Queen Mary University of London
%               Mile End Road
%               London - E1 4NS
%               United Kingdom
% Email: pft@spevi.org
% Citation: T. Nawaz, A. Cavallaro, PFT: A protocol for evaluating video trackers, in Proc. of IEEE Int. Conf. on Image Processing, Brussels, 11-14 September 2011
%
% Please note that this software is distributed under the terms and
% conditions of the creative commons public license (refer to the file 
% 'License.doc for more information).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;

fid = fopen('..\InputFile.txt');
if (fid>=3)
    disp('Input file opened successfully');
    C = textscan(fid,'%s');%Reading the parameters from the input file
    fclose(fid);
    fileParamInd  = 1;

    %Checking whether the tool is to be used in Input Generation (IG) Mode
    if (strcmpi(C{1}{fileParamInd}, 'IG-Mode'))
        fileParamInd = fileParamInd + 1;
        trial(1).name = lower(C{1}{fileParamInd}); % Name of the Trial to be used to evaluate performance of candidate tracker on
        fileParamInd = fileParamInd + 1;
        switch (trial(1).name)
            %settings for different trials
            
            %Trial in which both the video sequence and tracker's initialization
            %remains undisturbed
            case 'pft-p0'
                trial(1).numOfOptions = 1;
                trial(1).options = 'default';%selecting the operation to be performed in this trial;
                                             %in this case, no change to video sequence and initialization
                initFlag = 'false';

            %Trial in which perturbation is added to the position of
            %initialization only while keeping the its size (width, height) unchanged
            case 'pft-p1' 
                trial(1).numOfOptions = 1;
                initParam = getInitParamVidSeq();
                trial(1).options = 'initPertXY';%selecting the operation to be performed in this trial
                trial(1).numOfPerts = 20;%Number of perturbed initializations to be generated on this trial
                initFlag = 'true';

            %Trial in which perturbation is added to the size of
            %initialization bounding box only while keeping the its position unchanged
            case 'pft-p2'                    
                trial(1).numOfOptions = 1;
                initParam = getInitParamVidSeq();
                trial(1).options = 'initPertWH';%selecting the operation to be performed in this trial
                trial(1).numOfPerts = 20;
                initFlag = 'true';

            %Trial in which perturbation is added both to the size and
            %position of initialization bounding box
            case 'pft-p3'
                trial(1).numOfOptions = 1;
                initParam = getInitParamVidSeq();
                trial(1).options = 'initPertXYWH'; %selecting the operation to be performed in this trial
                trial(1).numOfPerts = 20;
                initFlag = 'true';
            
            %Trial in which different levels of noise are added to video
            %sequence
            case 'pft-p4'
                trial(1).numOfOptions = 3;%It tells that the operation is performed 3 times to
                                          %generate three new video sequences
                trial(1).options = 'addNoise';%selecting the operation to be performed in this trial
                trial(2).options = 'addNoise';
                trial(3).options = 'addNoise';
                m = [2 4 6];
                initFlag = 'false';

            %Trial which involves frame dropping/skipping
            case 'pft-p5'
                trial(1).numOfOptions = 4;%It tells that the operation is performed 4 times to
                                          %generate four new video sequences
                trial(1).options = 'dropFrames';
                trial(2).options = 'dropFrames';
                trial(3).options = 'dropFrames';
                trial(4).options = 'dropFrames';
                m = [2 4 6 8];%amounts of frame skipping. e.g. m=4 means that the next frame to be 
                              %processed is the (m+current frame)
                initFlag = 'false';
            
            %Trial which involves illumination change scenarios (linear increase/decrease
            %in the illumination along the frames of sequence)
            case 'pft-p6'
                trial(1).numOfOptions = 2;
                trial(1).options = 'illumChange';
                trial(2).options = 'illumChange';
                m = [1 2];% '1' represents increase in the illumination and 
                          % '2' represents decrease in the illumination of
                          % the sequence
                initFlag = 'false';                          
                
            otherwise
                error('Invalid trial name entered!');                
                
        end
        
        if (strcmp(initFlag, 'true'))        
            disp('Parameters of the original initializing bounding box are:');
            disp(['x=',num2str(initParam(1))]);
            disp(['y=',num2str(initParam(2))]);
            disp(['Width=',num2str(initParam(3))]);
            disp(['Height=',num2str(initParam(4))]);
        end
            
        %Executing operation depending upon the choice of a trial above
        for k = 1:trial(1).numOfOptions
            option = trial(k).options; % name of option/operation to be executed
            switch (option)                
               
                %Adding Gaussian noise to the given video sequence
                case 'addNoise'
                    if (k == 1)
                        disp('Trial-4 (PFT-P4): Addition of Gaussian noise to the video sequence');
                        disp('Generating the First video sequence after adding Gaussian noise with twice the modelled variance');
                    elseif (k == 2)
                        disp('Generating the Second video sequence after adding Gaussian noise with four times the modelled variance');
                    elseif (k == 3)
                        disp('Generating the Third video sequence after adding Gaussian noise with six times the modelled variance');
                    end
                                        
                    file_name = strcat('PFT_P4_',num2str(m(k)),'.avi');
                    AddGaussianNoiseFrames(file_name, m(k));
                    
                %Adding perturbations to the position of tracker's initialization
                case 'initPertXY'
                    disp('Trial-1 (PFT-P1): Addition of perturbations to the position of tracker''s initialization');
                    Perturbations = zeros(trial(1).numOfPerts,size(initParam,2));
                    initArea = initParam(3)*initParam(4);
                    Perturbations(:,3) = initParam(3);
                    Perturbations(:,4) = initParam(4);
                    mmReaderObj = mmreader('..\temp\VideoSequence.avi');
                    frameSize = [mmReaderObj.Height, mmReaderObj.Width];
                    firstFrame = read(mmReaderObj,1);
                    clear mmReaderObj;
                    amountPreservedPixels = 0.50;% For a perturbed initialization to be valid,
                                                 %it is ensured to have a 50 percent overlap between
                                                 %the original initialization and itself
                    disp('Generating the perturbed initializations');
                    for i = 1:trial(1).numOfPerts
                        if (rem(i,20)==0)
                            fprintf('.\n');
                        else
                            fprintf('.');
                        end                        
                        amount1 = initParam(3); 
                        amount2 = initParam(4);
                        overlap = 0;
                        while(overlap<=amountPreservedPixels)
                            amount1 = rand*(initParam(3));
                            amount2 = rand*(initParam(4));
                            if (rand>0.5)
                                Perturbations(i,1) = round(initParam(1) + amount1);
                            else
                                Perturbations(i,1) = round(initParam(1) - amount1);
                            end

                            if (rand>0.5)
                                Perturbations(i,2) = round(initParam(2) + amount2);
                            else
                                Perturbations(i,2) = round(initParam(2) - amount2);
                            end
                            %ensuring that new generated perturbed
                            %initialization remains within the
                            %boundary of the image
                            if ( (Perturbations(i,1)-round(initParam(3)/2))<=0 || ...
                                 (Perturbations(i,1)+round(initParam(3)/2))>frameSize(2) || ...
                                 (Perturbations(i,2)-round(initParam(4)/2))<=0 || ...
                                 (Perturbations(i,2)+round(initParam(4)/2))>frameSize(1) )
                                overlap = 0;
                            else
                                overlap = findOverlap(Perturbations(i,:),initParam,frameSize);
                            end
                        end
                    end
                    fprintf('\n');
                    overlayInitPerts(firstFrame, Perturbations);
                    initFileName = 'PertInits_Trial1.mat';
                    temp_location=strcat('''\PFT\InputForTracker\',initFileName,'''');
                    save ..\InputForTracker\PertInits_Trial1.mat Perturbations;
                    disp(['Perturbed initializations generated and saved at ',temp_location]);

                case 'initPertWH'
                    disp('Trial-2 (PFT-P2): Addition of perturbations to the size (width, height) of tracker''s initialization');
                    Perturbations = zeros(trial(1).numOfPerts,size(initParam,2));
                    initArea = initParam(3)*initParam(4);
                    Perturbations(:,1) = initParam(1);
                    Perturbations(:,2) = initParam(2);
                    mmReaderObj = mmreader('..\temp\VideoSequence.avi');
                    frameSize = [mmReaderObj.Height, mmReaderObj.Width];
                    firstFrame = read(mmReaderObj,1);
                    clear mmReaderObj;
                    amountPreservedPixels = 0.50;% For a perturbed initialization to be valid,
                                                 %it is ensured to have a 50 percent overlap between
                                                 %the original initialization and itself
                    disp('Generating the perturbed initializations');
                    for i = 1:trial(1).numOfPerts
                        if (rem(i,20)==0)
                            fprintf('.\n');
                        else
                            fprintf('.');
                        end
                        amount1 = initParam(3); 
                        amount2 = initParam(4);
                        pertInitArea = 0;
                        overlap = 0;
                        while(overlap<=amountPreservedPixels)
                            amount1 = rand*(initParam(3));
                            amount2 = rand*(initParam(4));
                            if (rand>0.5)
                                Perturbations(i,3) = round(initParam(3) + amount1);
                            else
                                Perturbations(i,3) = round(initParam(3) - amount1);
                            end

                            if (rand>0.5)
                                Perturbations(i,4) = round(initParam(4) + amount2);
                            else
                                Perturbations(i,4) = round(initParam(4) - amount2);
                            end
                            %ensuring that new generated perturbed
                            %initialization remains within the
                            %boundary of the image
                            if ( (Perturbations(i,1)-round(Perturbations(i,3)/2))<=0 || ...
                                 (Perturbations(i,1)+round(Perturbations(i,3)/2))>frameSize(2) || ...
                                 (Perturbations(i,2)-round(Perturbations(i,4)/2))<=0 || ...
                                 (Perturbations(i,2)+round(Perturbations(i,4)/2))>frameSize(1) )

                                overlap=0;
                            else
                                overlap = findOverlap(Perturbations(i,:),initParam,frameSize);
                            end
                        end
                    end
                    fprintf('\n');
                    overlayInitPerts(firstFrame, Perturbations);
                    initFileName = 'PertInits_Trial2.mat';
                    temp_location=strcat('''\PFT\InputForTracker\',initFileName,'''');
                    save ..\InputForTracker\PertInits_Trial2.mat Perturbations;
                    disp(['Perturbed initializations generated and saved at ',temp_location]);
                    
                case 'initPertXYWH'
                    disp('Trial-3 (PFT-P3): Addition of perturbations both to the position and size of tracker''s initialization');
                    Perturbations = zeros(trial(1).numOfPerts,size(initParam,2));
                    initArea = initParam(3)*initParam(4);
                    amountPreservedPixels = 0.50;% For a perturbed initialization to be valid,
                                                 %it is ensured to have a 50 percent overlap between
                                                 %the original initialization and itself
                    mmReaderObj = mmreader('..\temp\VideoSequence.avi');
                    frameSize = [mmReaderObj.Height, mmReaderObj.Width];
                    firstFrame = read(mmReaderObj,1);
                    clear mmReaderObj;
                    disp('Generating the perturbed initializations');
                    for i = 1:trial(1).numOfPerts
                        if (rem(i,20)==0)
                            fprintf('.\n');
                        else
                            fprintf('.');
                        end
                        overlap = 0;
                        while( (overlap<=amountPreservedPixels) )
                            amount1 = rand*(initParam(3));
                            amount2 = rand*(initParam(4));
                            amount3 = rand*(initParam(3));
                            amount4 = rand*(initParam(4));

                            if (rand>0.5)
                                Perturbations(i,1) = round(initParam(1) + amount1);
                            else
                                Perturbations(i,1) = round(initParam(1) - amount1);
                            end

                            if (rand>0.5)
                                Perturbations(i,2) = round(initParam(2) + amount2);
                            else
                                Perturbations(i,2) = round(initParam(2) - amount2);
                            end

                            if (rand>0.5)
                                Perturbations(i,3) = round(initParam(3) + amount3);                                    
                            else
                                Perturbations(i,3) = round(initParam(3) - amount3);
                                amount3 = -amount3;
                            end

                            if (rand>0.5)
                                Perturbations(i,4) = round(initParam(4) + amount4);
                            else
                                Perturbations(i,4) = round(initParam(4) - amount4);
                                amount4 = -amount4;
                            end

                            %ensuring that new generated perturbed
                            %initialization remains within the
                            %boundary of the image
                            if ( (Perturbations(i,1)-round(Perturbations(i,3)/2))<=0 || ...
                                 (Perturbations(i,1)+round(Perturbations(i,3)/2))>frameSize(2) || ...
                                 (Perturbations(i,2)-round(Perturbations(i,4)/2))<=0 || ...
                                 (Perturbations(i,2)+round(Perturbations(i,4)/2))>frameSize(1) )

                                overlap = 0;
                            else
                                overlap = findOverlap(Perturbations(i,:),initParam,frameSize);

                            end
                        end
                    end
                    fprintf('\n');
                    overlayInitPerts(firstFrame, Perturbations);
                    initFileName = 'PertInits_Trial3.mat';
                    temp_location=strcat('''\PFT\InputForTracker\',initFileName,'''');
                    save ..\InputForTracker\PertInits_Trial3.mat Perturbations;
                    disp(['Perturbed initializations generated and saved at ',temp_location]);

                case 'dropFrames'
                    if (k == 1)
                        disp('Trial-5 (PFT-P5): Frame skipping from the video sequence');
                        disp('Generating the First video sequence by retaining every ''k+2'' frame of it (where ''k'' is the current frame)');
                    elseif (k == 2)
                        disp('Generating the Second video sequence by retaining every ''k+4'' frame of it (where ''k'' is the current frame)');
                    elseif (k == 3)
                        disp('Generating the Third video sequence by retaining every ''k+6'' frame of it (where ''k'' is the current frame)');
                    elseif (k == 4)
                        disp('Generating the Fourth video sequence by retaining every ''k+8'' frame of it (where ''k'' is the current frame)');
                    end
                    
                    file_name = strcat('PFT_P5_',num2str(m(k)),'.avi');
                    DropFramesVideo(file_name, m(k));

                case 'illumChange'
                    if (k == 1)
                        disp('Trial-6 (PFT-P6): Changing illumination across the video sequence');
                        disp('Generating the First video sequence by increasing the illumination across the video sequence linearly');
                    elseif (k == 2)
                        disp('Generating the Second video sequence by decreasing the illumination across the video sequence linearly');
                    end
                    
                    if (m(k) == 1)
                        file_name = strcat('PFT_P6_incr.avi');
                        illumType = 'increase';
                    elseif (m(k) == 2)
                        file_name = strcat('PFT_P6_decr.avi');
                        illumType = 'decrease';
                    end
                    changeVidIllumination(file_name, illumType);
                                        
                case 'default'
                    disp('Trial-0 (PFT-P0): No disturbance added both to the tracker''s initialization and the video sequence');
                    disp('Test the tracker on the original video sequence and initialization');
            end
        end
        
    %Checking whether the tool is to be used in Evaluation (EVL) Mode    
    elseif (strcmpi(C{1}{fileParamInd}, 'EVL-Mode'))
        fileParamInd = fileParamInd + 1;
        protocol(1).name = lower(C{1}{fileParamInd}); % Name of trial
        fileParamInd = fileParamInd + 1;
        switch (protocol(1).name)
            case 'pft-p0'
                disp('Evaluation of tracker''s performance on Trial-0 (PFT-P0) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                plot_color = [0,0,1;
                              0,1,0;
                              1,0,0];
                if ((size(C{1},1)-2) == 0)
                    error('Tracker''s result file names not specified!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    PlotEvaluationScores(evaluationScores, plot_color(i,:),[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 0');
                figure(1);title(fig_title);
                figure(2);title(fig_title);

            case 'pft-p1'
                disp('Evaluation of tracker''s performance on Trial-1 (PFT-P1) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                numPert = (size(C{1},1)-2);
                if ((size(C{1},1)-2) == 0)
                    error('Tracker''s result file names not specified!');
                elseif ((size(C{1},1)-2) ~= 20)
                        error('Total number of tracker''s result file name(s) should be 20!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;

                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    plot_color = [rand, rand, rand];                    
                    PlotEvaluationScores(evaluationScores, plot_color,[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 1. Result for each perturbed initialization shown by randomly selected color.');
                figure(1);title(fig_title);
                figure(2);title(fig_title);
                figure(3); hold on;
                plot(1:numPert,AUC_lambda(1:end-2),'LineWidth',2,'Color', [0 0 1]);
                xlabel('Perturbed initializations'); ylabel('AUC_{\lambda}');
                xlim([1 numPert]); title('Trial 1');
                disp('AUC_lambda scores of tracker for all perturbed initializations plotted');

            case 'pft-p2'
                disp('Evaluation of tracker''s performance on Trial-2 (PFT-P2) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                numPert = (size(C{1},1)-2);
                if ((size(C{1},1)-2) == 0)
                    error('Tracker''s result file names not specified!');
                elseif ((size(C{1},1)-2) ~= 20)
                        error('Total number of tracker''s result file name(s) should be 20!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)                    
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);                    
                    plot_color = [rand, rand, rand];                    
                    PlotEvaluationScores(evaluationScores, plot_color,[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean'; 
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';                        
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 2 with target. Result for each perturbed initialization shown by randomly selected color.');
                figure(1);title(fig_title);
                figure(2);title(fig_title);
                figure(3); hold on;
                plot(1:numPert,AUC_lambda(1:end-2),'LineWidth',2,'Color', [0 0 1]);
                xlabel('Perturbed initializations'); ylabel('AUC_{\lambda}');
                xlim([1 numPert]); title('Trial 2');
                disp('AUC_lambda scores of tracker for all perturbed initializations plotted');

            case 'pft-p3'
                disp('Evaluation of tracker''s performance on Trial-3 (PFT-P3) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                numPert = (size(C{1},1)-2);
                if ((size(C{1},1)-2) == 0)
                    error('Tracker''s result file names not specified!');
                elseif ((size(C{1},1)-2) ~= 20)
                        error('Total number of tracker''s result file name(s) should be 20!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)                    
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    plot_color = [rand, rand, rand];
                    PlotEvaluationScores(evaluationScores, plot_color,[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 3. Result for each perturbed initialization shown by randomly selected color.');
                figure(1);title(fig_title);
                figure(2);title(fig_title);
                figure(3); hold on;
                plot(1:numPert,AUC_lambda(1:end-2),'LineWidth',2,'Color', [0 0 1]);
                xlabel('Perturbed initializations'); ylabel('AUC_{\lambda}');
                xlim([1 numPert]); title('Trial 3');
                disp('AUC_lambda scores of tracker for all perturbed initializations plotted');

            case 'pft-p4'
                disp('Evaluation of tracker''s performance on Trial-4 (PFT-P4) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                plot_color = [0,0,1;
                              0,1,0;
                              1,0,0;
                              0,0,0];
                          
                if ((size(C{1},1)-2) == 0)
                    error('Tracker''s result file names not specified!');
                elseif ((size(C{1},1)-2) ~= 3)
                        error('Total number of tracker''s result file name(s) should be three!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)                    
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    PlotEvaluationScores(evaluationScores, plot_color(i,:),[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 4');
                figure(1);title(fig_title);
                figure(1);legend('Gaussian noise = 2 x estimated variance with zero mean','Gaussian noise = 4 x estimated variance with zero mean','Gaussian noise = 6 x estimated variance with zero mean');
                figure(2);title(fig_title);
                figure(2);legend('Gaussian noise = 2 x estimated variance with zero mean','Gaussian noise = 4 x estimated variance with zero mean','Gaussian noise = 6 x estimated variance with zero mean');

            case 'pft-p5'
                disp('Evaluation of tracker''s performance on Trial-5 (PFT-P5) ...');
                fprintf('\n');                
                m = [2,4,6,8];
                plot_color = [0,0,1;
                              0,1,0;
                              1,0,0;
                              0,0,0];

                for k=1:size(m,2)
                    %Loading the ground truth of sequence
                    GroundTruth = getGroundTruth();
                    frameSize = getFrameSize();
                    %storing the indices of the retained frames (rest are dropped from the sequence)
                    origFrameInd = 1:m(k):size(GroundTruth,1);
                    frameInd = 0;
                    %Rataining ground truth of the frames which are not
                    %dropped
                    for i=1:m(k):size(GroundTruth,1)
                        frameInd = frameInd+1;
                        GT(frameInd,:) = GroundTruth(i,:);
                    end
                    GT(:,1) = 1:size(GT,1);
                    GroundTruth = GT;
                    clear GT;
                    
                    if ((size(C{1},1)-2) == 0)
                        error('Tracker''s result file names not specified!');
                    elseif ((size(C{1},1)-2) ~= 4)
                        error('Total number of tracker''s result file name(s) should be four!');
                    end
                
                    f_names{1}{k} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{k}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)                    
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    PlotEvaluationScores(evaluationScores, plot_color(k,:), origFrameInd);
                    AUC_lambda(k) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 5');
                figure(1);title(fig_title);
                figure(1);legend('m=2','m=4','m=6','m=8');
                figure(2);title(fig_title);
                figure(2);legend('m=2','m=4','m=6','m=8');

            case 'pft-p6'
                disp('Evaluation of tracker''s performance on Trial-6 (PFT-P6) ...');
                fprintf('\n');
                %Loading the ground truth of sequence
                GroundTruth = getGroundTruth();
                frameSize = getFrameSize();
                plot_color = [0,0,1;
                              0,1,0;
                              1,0,0];
                
                if ((size(C{1},1)-2) == 0)
                        error('Tracker''s result file names not specified!');
                elseif ((size(C{1},1)-2) ~= 2)
                        error('Total number of tracker''s result file name(s) should be two!');
                end
                for i = 1:(size(C{1},1)-2)
                    f_names{1}{i} = C{1}{fileParamInd};
                    TkrRslt_fileName = strcat('..\TrackerResults\',C{1}{fileParamInd});
                    %Loading tracker's results one by one
                    disp(['Loading the Tracker''s result file (',TkrRslt_fileName,') ...']);
                    Tkr_Results = convertTextFileData2Matrix(TkrRslt_fileName);
                    fileParamInd = fileParamInd + 1;
                    disp(['Computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda) for the tracker''s result: ', f_names{1}{i}]);
                    % computing evaluation scores (overlap measure, lost track ratio curve, AUC_lambda)                    
                    evaluationScores = computeEvaluationScores(Tkr_Results, GroundTruth, frameSize);
                    PlotEvaluationScores(evaluationScores, plot_color(i,:),[]);
                    AUC_lambda(i) = evaluationScores.AUC;
                end
                AUC_lambda = [AUC_lambda, mean(AUC_lambda), std(AUC_lambda)];
                f_names{1}{size(f_names{1},2)+1} = 'Mean';
                f_names{1}{size(f_names{1},2)+1} = 'Standard deviation';
                SaveEvaluationScores(AUC_lambda, f_names, []);
                fig_title = strcat('Evaluation results of tracker on Trial 6');
                figure(1);title(fig_title);
                figure(1);legend('+ \Delta L', '- \Delta L');
                figure(2);title(fig_title);
                figure(2);legend('+ \Delta L', '- \Delta L');
                
            otherwise
                error('Invalid trial name entered!');
                
        end
        
    else
        error('Invalid mode specified in Input file! This software can be used either in IG-Mode or EVL-Mode.');
    end
  
elseif (fid == -1)
    disp('Can not open the Input file');
    
end
